"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Credential = exports.Wallet = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

exports.verifySignature = verifySignature;
exports.verifyHexSignature = verifyHexSignature;
exports.verifyUTF8Signature = verifyUTF8Signature;
exports.createCredential = createCredential;
exports.credentialFromHex = credentialFromHex;
exports.signingKeyToAddress = signingKeyToAddress;
exports.verifyKeyToAddress = verifyKeyToAddress;

var _immutable = require("immutable");

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var sha256 = require("crypto-js/sha256");
var nacl_factory = require("js-nacl");
var nacl = nacl_factory.instantiate();
var Base58 = require("base-58");
var _ = require('lodash');

// Need to switch to bs58 or get base-58 to include an MIT License
// As it was ported from an MIT Licensed project, I'll make a pull request

// Object for holding a collection of addresses and their private keys.
// Will need to enable persistence of credentials across sessions.
// For example will need to save to file for desktop clients, but Local Storage
// for things like Chrome Extensions.
// An encryption layer will be needed as well.

var Wallet = exports.Wallet = function () {
  function Wallet(credentials, sigCredentials) {
    _classCallCheck(this, Wallet);

    this.credentials = credentials;
    this.sigCredentials = sigCredentials;
    if (this.credentials === undefined) {
      this.credentials = (0, _immutable.Map)();
    }
    if (this.sigCredentials === undefined) {
      this.sigCredentials = (0, _immutable.Map)();
    }
  }

  _createClass(Wallet, [{
    key: "createCredential",
    value: function createCredential() {
      var cred = new Credential();
      var address = cred.address;
      this.credentials = this.credentials.update(address, function (addr) {
        return addr = cred;
      });
      return cred;
    }
  }, {
    key: "createSigCredential",
    value: function createSigCredential() {
      var cred = new Credential();
      var address = cred.address;
      this.sigCredentials = this.sigCredentials.update(address, function (addr) {
        return addr = cred;
      });
      return cred;
    }
  }, {
    key: "signTransaction",
    value: function signTransaction(tx) {
      // Stub for a wallet to be able to take a transaction, determine if it's one
      // they can sign, then return the signed transaction if so.
      return false;
    }
  }, {
    key: "toJson",
    value: function toJson() {
      jsoned = { credentials: {}, sigCredentials: {} };

      // Append basic information to JSON object
      _.forEach(this.credentials, function (cred) {
        jsoned.credentials[cred.address] = {
          address: cred.address,
          signingKey: cred.skToHex(),
          verifyKey: cred.vkToHex()
        };
      });
      // Same thing for Sig Credentials
      _.forEach(this.sigCredentials, function (cred) {
        jsoned.sigCredentials[cred.address] = {
          address: cred.address,
          signingKey: cred.skToHex(),
          verifyKey: cred.vkToHex()
        };
      });

      return JSON.stringify(jsoned);
    }
  }, {
    key: "fromJson",
    value: function fromJson(jsonString) {
      this.credentials = [];
      this.sigCredentials = [];
      if (jsonString === 'string') {
        jsoned = JSON.parse(jsonString);
      } else {
        jsoned = jsonString;
      }

      _.forEach(jsoned.credentials, function (cred) {
        this.credentials[cred.address] = credentialFromHex(cred.signingKey);
      });

      _.forEach(jsoned.sigCredentials, function (cred) {
        this.sigCredentials[cred.address] = credentialFromHex(cred.signingKey);
      });
    }
  }]);

  return Wallet;
}();

// Class for holding wallet information about a single address.


var Credential = exports.Credential = function () {

  // Constructor creates random

  function Credential(address, signingKey, verifyKey) {
    _classCallCheck(this, Credential);

    this.address = address;
    this.signingKey = signingKey;
    this.verifyKey = verifyKey;

    // Populate with random key if there isn't a populated set of credentials.
    if (signingKey === undefined && verifyKey === undefined) {
      this.signingKey = nacl.crypto_sign_keypair();
    }
    if (verifyKey === undefined) {
      this.verifyKey = this.signingKey.signPk;
    }
    if (address === undefined) {
      this.address = verifyKeyToAddress(nacl.to_hex(this.verifyKey));
    }
  }

  _createClass(Credential, [{
    key: "skToHex",
    value: function skToHex() {
      return nacl.to_hex(this.signingKey.signSk.slice(0, 32));
    }
  }, {
    key: "vkToHex",
    value: function vkToHex() {
      return nacl.to_hex(this.verifyKey);
    }
  }, {
    key: "sign",


    // Expects Uint8Array for a message
    // Need to make sure we're on same page for order of functions arguments and returns
    value: function sign(message) {
      var signature = nacl.crypto_sign_detached(message, this.signingKey.signSk);
      return [signature, message, this.signingKey.signPk];
    }
  }, {
    key: "signUTF8",

    // Doesn't really make sense to return the signature and pubkey in UTF8 format.
    value: function signUTF8(message) {
      var m = nacl.encode_utf8(message);
      var result = this.sign(m);
      var signature = nacl.to_hex(result[0]);
      var pubKey = nacl.to_hex(result[2]);
      return [signature, message, pubKey];
    }
  }, {
    key: "signHex",

    // Straightforward. Takes a Hex message and return signature, message,
    // and pubkey in hex format.
    value: function signHex(message) {
      var m = nacl.from_hex(message);
      var result = this.sign(m);
      var signature = nacl.to_hex(result[0]);
      var pubKey = nacl.to_hex(result[2]);
      return [signature, message, pubKey];
    }
  }]);

  return Credential;
}();

// Requires the msg, verifying key, and signature in Uint8Array format


function verifySignature(signature, msg, verifyKey) {
  var result = nacl.crypto_sign_verify_detached(signature, msg, verifyKey);
  return result;
}

// Requires the msg, verifying key, and signature in Hex format
function verifyHexSignature(signature, msg, verifyKey) {
  var s = nacl.from_hex(signature);
  var m = nacl.from_hex(msg);
  var v = nacl.from_hex(verifyKey);
  return verifySignature(s, m, v);
}

// Requires the msg in UTF8 and verifying key and signature in Hex format
function verifyUTF8Signature(s, m, v) {
  return verifySignature(nacl.from_hex(s), nacl.encode_utf8(m), nacl.from_hex(v));
}

function createCredential() {
  var k = nacl.crypto_sign_keypair();
  var cred = new Credential(signingKeyToAddress(k), k, k.signPk);
  return cred;
}

function credentialFromHex(signingKey) {
  var k = nacl.crypto_sign_keypair_from_seed(nacl.from_hex(signingKey));
  var cred = new Credential(signingKeyToAddress(k), k, k.signPk);
  return cred;
}

// Need to incorporate a known implementation into the testing suite
// Have verified it conforms to Kotlin implementation for at least one example
function signingKeyToAddress(k) {
  return verifyKeyToAddress(nacl.to_hex(k.signPk));
}

function verifyKeyToAddress(k) {
  var vkb = k;
  vkb = sha256(vkb).toString();
  vkb = "00" + vkb.substring(0, 40);
  var cs = sha256(sha256(vkb).toString()).toString().substring(0, 8);
  var address = vkb + cs;
  return Base58.encode(nacl.from_hex(address));
}