'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.SingleKeyProof = exports.Proof = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }(); /*
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       Implements a type of proof that allows an address to authorize a payment.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       Single Key Addresses are the initial supported type.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     */


var _credentials = require('./credentials.js');

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Proof = exports.Proof = function () {
  function Proof(proof) {
    _classCallCheck(this, Proof);

    if (proof === undefined) proof = {};
    if (proof.address === undefined) {
      this.address = '';
    } else {
      this.address = proof.address;
    }
    if (proof.type === undefined) {
      this.type = '';
    } else {
      this.type = proof.type;
    }
    if (proof.payload === undefined) {
      this.payload = {};
    } else {
      this.payload = proof.payload;
    }
    if (proof.metadata === undefined) {
      this.metadata = {};
    } else {
      this.metadata = proof.metadata;
    }
    if (proof.challenge === undefined) {
      this.challenge = '';
    } else {
      this.challenge = proof.challenge;
    }
  }

  _createClass(Proof, [{
    key: 'toJson',
    value: function toJson() {
      return {
        type: this.type,
        payload: this.payload,
        metadata: this.metadata
      };
    }
  }, {
    key: 'sign',
    value: function sign(credential) {
      if (credential.address != this.address) {
        return {
          error: "Invalid address for Proof"
        };
      } else {
        return {
          error: "Not Implemented"
        };
        // Implement proof-type specific signing methods by subclassing Proof.
      }
    }
  }, {
    key: 'verify',
    value: function verify() {
      // Implement proof-type specific verification methods by subclassing Proof.
      return {
        error: "Not Implemented"
      };
    }
  }]);

  return Proof;
}();

var SingleKeyProof = exports.SingleKeyProof = function (_Proof) {
  _inherits(SingleKeyProof, _Proof);

  function SingleKeyProof() {
    _classCallCheck(this, SingleKeyProof);

    return _possibleConstructorReturn(this, Object.getPrototypeOf(SingleKeyProof).apply(this, arguments));
  }

  _createClass(SingleKeyProof, [{
    key: 'verify',

    /*
    Initial supported Proof type. Allows Addresses created for use by a Single Signing Key.
     Sample SingleKeyProof Object as used in the Transaction Class.
    "proofs": {
      "1FG3LHD7kmQeZnCUXQJ756H2sGvAwiQCY": {
        "type": "works.credits.core.single_address",
        "payload": {
          "verify_key": "2cd32af3ec59cac5e1b6acb2a6ed4116de68aa68ffa242d43229a1ca0a4c6c1a",
          "signature": "4bacf4333680508ba51de3c0fbba41005a55d7076e3ef4657e544060ba0cf30a690ad42c0104e577ba1b0930d487f757a1f8fedec9301a0dd12be4cd0e58f20f"
        },
        "metadata": {}
      }
    }
    */
    value: function verify() {
      var errors = [];

      if (this.type != "works.credits.core.single_address") {
        errors.push("Incorrect fqdn for this Proof type.");
      }

      var verify_address = (0, _credentials.verifyKeyToAddress)(this.payload.verify_key);
      if (this.address != verify_address) {
        errors.push("Address Mismatch to Verify Key");
      }

      var verify_signature = (0, _credentials.verifyUTF8Signature)(this.payload.signature, this.challenge, this.payload.verify_key);
      if (!verify_signature) {
        errors.push("Incorrect Signature for Challenge");
      }

      var result = false;
      if (errors.length === 0) {
        result = true;
      }
      return {
        error: errors,
        result: result
      };
    }
  }]);

  return SingleKeyProof;
}(Proof);