'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Transaction = undefined;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }(); /*
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       Generic Transaction forming, signing, and verifying here.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       Need to place any network-specific transaction definitions elsewhere.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     */


exports.hashifyObject = hashifyObject;
exports.stringifyObject = stringifyObject;

var _lodash = require('lodash');

var _ = _interopRequireWildcard(_lodash);

var _proofs = require('./proofs.js');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var sha256 = require("crypto-js/sha256");


/*
  Sample TX Payload for simple use case.
  {
    "hash": "e076865b9e984447cb9f0aadf318a5cec909e4f07e400ddf644e9332a1b4c38d",
    "fqdn": "works.credits.balance",
    "network_id": "threashold-test-net",
    "type": 0,
    "data": {
      "nonce": 1310,
      "fromAddr": "1FG3LHD7kmQeZnCUXQJ756H2sGvAwiQCY",
      "amount": 100,
      "fee": 1,
      "toAddr":
      "bin"
    },
    "proofs": {
      "1FG3LHD7kmQeZnCUXQJ756H2sGvAwiQCY": {
        "type": "works.credits.core.single_address",
        "payload": {
          "verify_key": "2cd32af3ec59cac5e1b6acb2a6ed4116de68aa68ffa242d43229a1ca0a4c6c1a",
          "signature": "4bacf4333680508ba51de3c0fbba41005a55d7076e3ef4657e544060ba0cf30a690ad42c0104e577ba1b0930d487f757a1f8fedec9301a0dd12be4cd0e58f20f"
        },
        "metadata": {}
      }
    }
  }
*/

var Transaction = exports.Transaction = function () {
  function Transaction(tx) {
    _classCallCheck(this, Transaction);

    if (tx === undefined) tx = {};
    if (tx.fqdn === undefined) {
      this.fqdn = '';
    } else {
      this.fqdn = tx.fqdn;
    }
    if (tx.network_id === undefined) {
      this.network_id = '';
    } else {
      this.network_id = tx.network_id;
    }
    if (tx.type === undefined) {
      this.type = -1;
    } else {
      this.type = tx.type;
    }
    if (tx.data === undefined) {
      this.data = {};
    } else {
      this.data = tx.data;
    }
    this.proofs = {};
    if (tx.proofs !== undefined) {
      for (var proof in tx.proofs) {
        var proofObj = tx.proofs[proof];
        proofObj.address = proof;
        proofObj.challenge = this.hash();
        this.proofs[proof] = new _proofs.Proof(proofObj);
      }
    }
  }

  _createClass(Transaction, [{
    key: 'toJson',
    value: function toJson() {
      return {
        fqdn: this.fqdn,
        network_id: this.network_id,
        type: this.type,
        data: this.data,
        proofs: this.jsonProofs(),
        hash: this.hash()
      };
    }
  }, {
    key: 'jsonProofs',
    value: function jsonProofs() {
      var proofs = {};
      for (var proof in this.proofs) {
        proofs[proof] = this.proofs[proof].toJson();
      }
      return proofs;
    }
  }, {
    key: 'hash',
    value: function hash() {
      // Should return a Hex String
      /*
        This is generating the hash by hashing the key/value pairs from the data
        sorted by the key in alphanumerical order, separated by a delimiter.
      */
      var delimiter = '|';
      var source = {
        network_id: this.network_id,
        type: this.type,
        fqdn: this.fqdn,
        data: this.data
      };
      return hashifyObject(source, delimiter);
    }
  }, {
    key: 'getChallenge',
    value: function getChallenge(metadata) {
      /*
        This is a challenge for the Credential that needs to authorize a request needs to sign.
         In most cases, this will simply be the hash of the Transaction.
      */
      return this.hash();
    }
  }, {
    key: 'sign',
    value: function sign(credential) {
      // Should return a Hex String
      if (this.fromVk !== credential.vkToHex()) return false;
      var signature = credential.signUTF8(this.hash())[0];
      this.signature = signature;
      return signature;
    }
  }, {
    key: 'verify',
    value: function verify(StateView) {
      // Should return an array with [Result (Boolean), Error Code]
      // Needs to depend on TX type and have some way of registering TX types.
      return '';
    }
  }, {
    key: 'apply',


    // Stub
    value: function apply(StateView) {
      // Low on list of priorities to implement for initial clientside
      // Should return an array with [Result (Boolean), New StateView, Error Code]
      return '';
    }
  }]);

  return Transaction;
}();

function hashifyObject(obj, delimiter) {
  var hashString = stringifyObject(obj, delimiter);
  hashString = hashString.substring(0, hashString.length - 1);
  var hash = sha256(hashString).toString();
  return hash;
}

function stringifyObject(obj, delimiter) {
  var hashString = '';
  if (delimiter === undefined) delimiter = '|';
  if ((typeof obj === 'undefined' ? 'undefined' : _typeof(obj)) === 'object') {
    _.forEach(_.sortBy(_.keys(obj), function (o) {
      return o;
    }), function (key) {
      if (!Array.isArray(obj)) {
        hashString += new Buffer(key).toString('hex');
        hashString += delimiter;
      }
      hashString += stringifyObject(obj[key], delimiter);
    });
  } else if (typeof obj === 'number') {
    hashString += new Buffer(obj.toString(10)).toString('hex');
    hashString += delimiter;
  } else {
    hashString += new Buffer(obj).toString('hex');
    hashString += delimiter;
  }

  return hashString;
}